@@ -16,38 +16,39 @@ use super::{
     AllocRegResult, Env, LiveRangeKey, PReg, PRegIndex, RegTraversalIter, SpillSetIndex,
     SpillSlotData, SpillSlotIndex,
 };
-use crate::{ion::data_structures::SpillSetRanges, Allocation, Function, SpillSlot};
+use crate::{Allocation, Function, SpillSlot};
 
 impl<'a, F: Function> Env<'a, F> {
     pub fn try_allocating_regs_for_spilled_bundles(&mut self) {
         trace!("allocating regs for spilled bundles");
-        for i in 0..self.spilled_bundles.len() {
-            let bundle = self.spilled_bundles[i]; // don't borrow self
+        let mut scratch = core::mem::take(&mut self.ctx.scratch_conflicts);
+        for i in 0..self.ctx.spilled_bundles.len() {
+            let bundle = self.ctx.spilled_bundles[i]; // don't borrow self
 
-            if self.bundles[bundle].ranges.is_empty() {
+            if self.ctx.bundles[bundle].ranges.is_empty() {
                 continue;
             }
 
-            let class = self.spillsets[self.bundles[bundle].spillset].class;
-            let hint = self.spillsets[self.bundles[bundle].spillset].reg_hint;
+            let class = self.ctx.spillsets[self.ctx.bundles[bundle].spillset].class;
+            let hint = self.ctx.spillsets[self.ctx.bundles[bundle].spillset].reg_hint;
 
             // This may be an empty-range bundle whose ranges are not
             // sorted; sort all range-lists again here.
-            self.bundles[bundle]
+            self.ctx.bundles[bundle]
                 .ranges
                 .sort_unstable_by_key(|entry| entry.range.from);
 
             let mut success = false;
-            self.stats.spill_bundle_reg_probes += 1;
+            self.ctx.output.stats.spill_bundle_reg_probes += 1;
             for preg in
                 RegTraversalIter::new(self.env, class, hint, PReg::invalid(), bundle.index(), None)
             {
                 trace!("trying bundle {:?} to preg {:?}", bundle, preg);
                 let preg_idx = PRegIndex::new(preg.index());
                 if let AllocRegResult::Allocated(_) =
-                    self.try_to_allocate_bundle_to_reg(bundle, preg_idx, None)
+                    self.try_to_allocate_bundle_to_reg(bundle, preg_idx, None, &mut scratch)
                 {
-                    self.stats.spill_bundle_reg_success += 1;
+                    self.ctx.output.stats.spill_bundle_reg_success += 1;
                     success = true;
                     break;
                 }
@@ -56,11 +57,12 @@ impl<'a, F: Function> Env<'a, F> {
                 trace!(
                     "spilling bundle {:?}: marking spillset {:?} as required",
                     bundle,
-                    self.bundles[bundle].spillset
+                    self.ctx.bundles[bundle].spillset
                 );
-                self.spillsets[self.bundles[bundle].spillset].required = true;
+                self.ctx.spillsets[self.ctx.bundles[bundle].spillset].required = true;
             }
         }
+        self.ctx.scratch_conflicts = scratch;
     }
 
     pub fn spillslot_can_fit_spillset(
@@ -68,10 +70,12 @@ impl<'a, F: Function> Env<'a, F> {
         spillslot: SpillSlotIndex,
         spillset: SpillSetIndex,
     ) -> bool {
-        !self.spillslots[spillslot.index()]
+        !self.ctx.spillslots[spillslot.index()]
             .ranges
             .btree
-            .contains_key(&LiveRangeKey::from_range(&self.spillsets[spillset].range))
+            .contains_key(&LiveRangeKey::from_range(
+                &self.ctx.spillsets[spillset].range,
+            ))
     }
 
     pub fn allocate_spillset_to_spillslot(
@@ -79,10 +83,10 @@ impl<'a, F: Function> Env<'a, F> {
         spillset: SpillSetIndex,
         spillslot: SpillSlotIndex,
     ) {
-        self.spillsets[spillset].slot = spillslot;
+        self.ctx.spillsets[spillset].slot = spillslot;
 
-        let res = self.spillslots[spillslot.index()].ranges.btree.insert(
-            LiveRangeKey::from_range(&self.spillsets[spillset].range),
+        let res = self.ctx.spillslots[spillslot.index()].ranges.btree.insert(
+            LiveRangeKey::from_range(&self.ctx.spillsets[spillset].range),
             spillset,
         );
 
@@ -92,19 +96,20 @@ impl<'a, F: Function> Env<'a, F> {
     pub fn allocate_spillslots(&mut self) {
         const MAX_ATTEMPTS: usize = 10;
 
-        for spillset in 0..self.spillsets.len() {
+        for spillset in 0..self.ctx.spillsets.len() {
             trace!("allocate spillslot: {}", spillset);
             let spillset = SpillSetIndex::new(spillset);
-            if !self.spillsets[spillset].required {
+            if !self.ctx.spillsets[spillset].required {
                 continue;
             }
-            let class = self.spillsets[spillset].class as usize;
+            let class = self.ctx.spillsets[spillset].class as usize;
             // Try a few existing spillslots.
-            let mut i = self.slots_by_class[class].probe_start;
+            let mut i = self.ctx.slots_by_class[class].probe_start;
             let mut success = false;
             // Never probe the same element more than once: limit the
             // attempt count to the number of slots in existence.
-            for _attempt in 0..core::cmp::min(self.slots_by_class[class].slots.len(), MAX_ATTEMPTS)
+            for _attempt in
+                0..core::cmp::min(self.ctx.slots_by_class[class].slots.len(), MAX_ATTEMPTS)
             {
                 // Note: this indexing of `slots` is always valid
                 // because either the `slots` list is empty and the
@@ -113,43 +118,44 @@ impl<'a, F: Function> Env<'a, F> {
                 // in-bounds (because it is made so below when we add
                 // a slot, and it always takes on the last index `i`
                 // after this loop).
-                let spillslot = self.slots_by_class[class].slots[i];
+                let spillslot = self.ctx.slots_by_class[class].slots[i];
 
                 if self.spillslot_can_fit_spillset(spillslot, spillset) {
                     self.allocate_spillset_to_spillslot(spillset, spillslot);
                     success = true;
-                    self.slots_by_class[class].probe_start = i;
+                    self.ctx.slots_by_class[class].probe_start = i;
                     break;
                 }
 
-                i = self.slots_by_class[class].next_index(i);
+                i = self.ctx.slots_by_class[class].next_index(i);
             }
 
             if !success {
                 // Allocate a new spillslot.
-                let spillslot = SpillSlotIndex::new(self.spillslots.len());
-                self.spillslots.push(SpillSlotData {
-                    ranges: SpillSetRanges::new(),
+                let spillslot = SpillSlotIndex::new(self.ctx.spillslots.len());
+                self.ctx.spillslots.push(SpillSlotData {
+                    ranges: self.ctx.scratch_spillset_pool.pop().unwrap_or_default(),
                     alloc: Allocation::none(),
-                    slots: self.func.spillslot_size(self.spillsets[spillset].class) as u32,
+                    slots: self.func.spillslot_size(self.ctx.spillsets[spillset].class) as u32,
                 });
-                self.slots_by_class[class].slots.push(spillslot);
-                self.slots_by_class[class].probe_start = self.slots_by_class[class].slots.len() - 1;
+                self.ctx.slots_by_class[class].slots.push(spillslot);
+                self.ctx.slots_by_class[class].probe_start =
+                    self.ctx.slots_by_class[class].slots.len() - 1;
 
                 self.allocate_spillset_to_spillslot(spillset, spillslot);
             }
         }
 
         // Assign actual slot indices to spillslots.
-        for i in 0..self.spillslots.len() {
-            self.spillslots[i].alloc = self.allocate_spillslot(self.spillslots[i].slots);
+        for i in 0..self.ctx.spillslots.len() {
+            self.ctx.spillslots[i].alloc = self.allocate_spillslot(self.ctx.spillslots[i].slots);
         }
 
         trace!("spillslot allocator done");
     }
 
     pub fn allocate_spillslot(&mut self, size: u32) -> Allocation {
-        let mut offset = self.num_spillslots;
+        let mut offset = self.ctx.output.num_spillslots as u32;
         // Align up to `size`.
         debug_assert!(size.is_power_of_two());
         offset = (offset + size - 1) & !(size - 1);
@@ -159,7 +165,7 @@ impl<'a, F: Function> Env<'a, F> {
             offset
         };
         offset += size;
-        self.num_spillslots = offset;
+        self.ctx.output.num_spillslots = offset as _;
         Allocation::stack(SpillSlot::new(slot as usize))
     }
 }
